@@ -47,10 +47,6 @@ class MissingRequiredSource(ExitingException):
     pass
 
 
-class PermissionsError(ExitingException):
-    pass
-
-
 class UserNotFound(ExitingException):
     pass
 
@@ -61,34 +57,54 @@ class ConfigFileBadState(ExitingException):
 
 class ConfigFile(object):
 
-    def __init__(self, source, dest, owner, perm, optional=False):
+    def __init__(self, source, dest, owner=None, perm=None, optional=False,
+                 preserve_properties=False, merge=False):
         self.source = source
         self.dest = dest
         self.owner = owner
         self.perm = perm
         self.optional = optional
+        self.merge = merge
+        self.preserve_properties = preserve_properties
 
     def __str__(self):
         return '<ConfigFile source:"{}" dest:"{}">'.format(self.source,
                                                            self.dest)
 
-    def _copy_dir(self, source, dest):
-        LOG.info('Copying dir from %s to %s', source, dest)
-        shutil.copytree(source, dest)
-        for root, dirs, files in os.walk(dest):
-            for dir_ in dirs:
-                self._set_permission(os.path.join(root, dir_))
-            for file_ in files:
-                self._set_permission(os.path.join(root, file_))
+    def _copy_file(self, source, dest):
+        self._delete_path(dest)
+        # dest endswith / means copy the <source> to <dest> folder
+        LOG.info('Copying %s to %s', source, dest)
+        if self.merge and self.preserve_properties and os.path.islink(source):
+            link_target = os.readlink(source)
+            os.symlink(link_target, dest)
+        else:
+            shutil.copy(source, dest)
+            self._set_properties(source, dest)
+
+    def _merge_directories(self, source, dest):
+        if os.path.isdir(source):
+            if os.path.lexists(dest) and not os.path.isdir(dest):
+                self._delete_path(dest)
+            if not os.path.isdir(dest):
+                LOG.info('Creating directory %s', dest)
+                os.makedirs(dest)
+            self._set_properties(source, dest)
+
+            dir_content = os.listdir(source)
+            for to_copy in dir_content:
+                self._merge_directories(os.path.join(source, to_copy),
+                                        os.path.join(dest, to_copy))
+        else:
+            self._copy_file(source, dest)
 
     def _delete_path(self, path):
-        if not os.path.exists(path):
+        if not os.path.lexists(path):
             return
+        LOG.info('Deleting %s', path)
         if os.path.isdir(path):
-            LOG.info('Deleting dir %s', path)
             shutil.rmtree(path)
         else:
-            LOG.info('Deleting file %s', path)
             os.remove(path)
 
     def _create_parent_dirs(self, path):
@@ -96,26 +112,21 @@ class ConfigFile(object):
         if not os.path.exists(parent_path):
             os.makedirs(parent_path)
 
-    def _copy_file(self, source, dest):
-        # dest endswith / means copy the <source> to <dest> folder
-        LOG.info('Coping file from %s to %s', source, dest)
-        shutil.copy(source, dest)
-        self._set_permission(dest)
+    def _set_properties(self, source, dest):
+        if self.preserve_properties:
+            self._set_properties_from_file(source, dest)
+        else:
+            self._set_properties_from_conf(dest)
 
-    def _set_permission(self, path):
-        user = pwd.getpwnam(self.owner)
-        uid, gid = user.pw_uid, user.pw_gid
-        LOG.info('Setting file %s owner to %s:%s', path,
-                 self.owner, self.owner)
-        os.chown(path, uid, gid)
-        # NOTE(Jeffrey4l): py3 need '0oXXX' format for octal literals, and py2
-        # support such format too.
-        perm = self.perm
-        if len(perm) == 4 and perm[1] != 'o':
-            perm = ''.join([perm[:1], 'o', perm[1:]])
-        perm = int(perm, base=0)
-        LOG.info('Setting file %s permission to %s', path, self.perm)
-        os.chmod(path, perm)
+    def _set_properties_from_file(self, source, dest):
+        shutil.copystat(source, dest)
+        stat = os.stat(source)
+        os.chown(dest, stat.st_uid, stat.st_gid)
+
+    def _set_properties_from_conf(self, path):
+        config = {'permissions':
+                  [{'owner': self.owner, 'path': path, 'perm': self.perm}]}
+        handle_permissions(config)
 
     def copy(self):
 
@@ -133,12 +144,10 @@ class ConfigFile(object):
             # otherwise means copy the source to dest
             if dest.endswith(os.sep):
                 dest = os.path.join(dest, os.path.basename(source))
-            self._delete_path(dest)
+            if not self.merge:
+                self._delete_path(dest)
             self._create_parent_dirs(dest)
-            if os.path.isdir(source):
-                self._copy_dir(source, dest)
-            else:
-                self._copy_file(source, dest)
+            self._merge_directories(source, dest)
 
     def _cmp_file(self, source, dest):
         # check exsit
@@ -160,15 +169,16 @@ class ConfigFile(object):
                       self.perm, actual_perm)
             return False
         # check owner
+        desired_user, desired_group = user_group(self.owner)
         actual_user = pwd.getpwuid(file_stat.st_uid)
-        if actual_user.pw_name != self.owner:
+        if actual_user.pw_name != desired_user:
             LOG.error('Dest file does not have expected user: %s,'
-                      ' actual: %s ', self.owner, actual_user.pw_name)
+                      ' actual: %s ', desired_user, actual_user.pw_name)
             return False
         actual_group = grp.getgrgid(file_stat.st_gid)
-        if actual_group.gr_name != self.owner:
+        if actual_group.gr_name != desired_group:
             LOG.error('Dest file does not have expected group: %s,'
-                      ' actual: %s ', self.owner, actual_group.gr_name)
+                      ' actual: %s ', desired_group, actual_group.gr_name)
             return False
         return True
 
@@ -182,7 +192,7 @@ class ConfigFile(object):
                 actual_perm = oct(dir_stat.st_mode)[-4:]
                 if self.perm != actual_perm:
                     LOG.error('Dest dir does not have expected perm: %s,'
-                              ' acutal %s', self.perm, actual_perm)
+                              ' actual %s', self.perm, actual_perm)
                     return False
             for file_ in files:
                 full_path = os.path.join(root, file_)
@@ -217,7 +227,7 @@ class ConfigFile(object):
 
 
 def validate_config(config):
-    required_keys = {'source', 'dest', 'owner', 'perm'}
+    required_keys = {'source', 'dest'}
 
     if 'command' not in config:
         raise InvalidConfig('Config is missing required "command" key')
@@ -228,6 +238,10 @@ def validate_config(config):
         if not data.viewkeys() >= required_keys:
             message = 'Config is missing required keys: %s' % required_keys
             raise InvalidConfig(message)
+        if ('owner' not in data or 'perm' not in data) \
+                and not data.get('preserve_properties', False):
+            raise InvalidConfig(
+                'Config needs preserve_properties or owner and perm')
 
 
 def validate_source(data):
@@ -246,42 +260,6 @@ def validate_source(data):
     return True
 
 
-def set_permissions(data):
-    def set_perms(file_, uid, guid, perm):
-        LOG.info("Setting permissions for %s", file_)
-        # Give config file proper perms.
-        try:
-            os.chown(file_, uid, gid)
-            os.chmod(file_, perm)
-        except OSError as e:
-            raise PermissionsError(
-                "Error while setting permissions for %s: %r" % (file_,
-                                                                repr(e)))
-
-    dest = data.get('dest')
-    owner = data.get('owner')
-    perm = int(data.get('perm'), 0)
-
-    # Check for user and group id in the environment.
-    try:
-        user = pwd.getpwnam(owner)
-    except KeyError:
-        raise UserNotFound("The specified user does not exist: %s" % owner)
-
-    uid = user.pw_uid
-    gid = user.pw_gid
-
-    # Set permissions on the top level dir or file
-    set_perms(dest, uid, gid, perm)
-    if os.path.isdir(dest):
-        # Recursively set permissions
-        for root, dirs, files in os.walk(dest):
-            for dir_ in dirs:
-                set_perms(os.path.join(root, dir_), uid, gid, perm)
-            for file_ in files:
-                set_perms(os.path.join(root, file_), uid, gid, perm)
-
-
 def load_config():
     def load_from_env():
         config_raw = os.environ.get("KOLLA_CONFIG")
@@ -336,34 +314,60 @@ def copy_config(config):
         f.write(config['command'])
 
 
+def user_group(owner):
+    if ':' in owner:
+        user, group = owner.split(':', 1)
+        if not group:
+            group = user
+    else:
+        user, group = owner, owner
+    return user, group
+
+
 def handle_permissions(config):
     for permission in config.get('permissions', list()):
         path = permission.get('path')
         owner = permission.get('owner')
         recurse = permission.get('recurse', False)
+        perm = permission.get('perm')
 
-        if ':' in owner:
-            user, group = owner.split(':', 1)
-            if not group:
-                group = user
-        else:
-            user, group = owner, owner
+        desired_user, desired_group = user_group(owner)
+        uid = pwd.getpwnam(desired_user).pw_uid
+        gid = grp.getgrnam(desired_group).gr_gid
 
-        uid = pwd.getpwnam(user).pw_uid
-        gid = grp.getgrnam(group).gr_gid
-
-        def set_perms(path, uid, gid):
+        def set_perms(path, uid, gid, perm):
             LOG.info('Setting permission for %s', path)
-            os.chown(path, uid, gid)
+            if not os.path.exists(path):
+                LOG.warning('%s does not exist', path)
+                return
+
+            try:
+                os.chown(path, uid, gid)
+            except OSError:
+                LOG.exception('Failed to change ownership of %s to %s:%s',
+                              path, uid, gid)
+
+            if perm:
+                # NOTE(Jeffrey4l): py3 need '0oXXX' format for octal literals,
+                # and py2 support such format too.
+                if len(perm) == 4 and perm[1] != 'o':
+                    perm = ''.join([perm[:1], 'o', perm[1:]])
+                perm = int(perm, base=0)
+
+                try:
+                    os.chmod(path, perm)
+                except OSError:
+                    LOG.exception('Failed to set permission of %s to %s',
+                                  path, perm)
 
         for dest in glob.glob(path):
-            set_perms(dest, uid, gid)
+            set_perms(dest, uid, gid, perm)
             if recurse and os.path.isdir(dest):
                 for root, dirs, files in os.walk(dest):
                     for dir_ in dirs:
-                        set_perms(os.path.join(root, dir_), uid, gid)
+                        set_perms(os.path.join(root, dir_), uid, gid, perm)
                     for file_ in files:
-                        set_perms(os.path.join(root, file_), uid, gid)
+                        set_perms(os.path.join(root, file_), uid, gid, perm)
 
 
 def execute_config_strategy(config):
@@ -392,28 +396,27 @@ def execute_config_check(config):
 
 
 def main():
-    parser = argparse.ArgumentParser()
-    parser.add_argument('--check',
-                        action='store_true',
-                        required=False,
-                        help='Check whether the configs changed')
-    args = parser.parse_args()
-    config = load_config()
+    try:
+        parser = argparse.ArgumentParser()
+        parser.add_argument('--check',
+                            action='store_true',
+                            required=False,
+                            help='Check whether the configs changed')
+        args = parser.parse_args()
+        config = load_config()
 
-    if args.check:
-        execute_config_check(config)
-    else:
-        execute_config_strategy(config)
+        if args.check:
+            execute_config_check(config)
+        else:
+            execute_config_strategy(config)
+    except ExitingException as e:
+        LOG.error("%s: %s", e.__class__.__name__, e)
+        return e.exit_code
+    except Exception:
+        LOG.exception('Unexpected error:')
+        return 2
+    return 0
 
 
 if __name__ == "__main__":
-    _exit_code = 0
-    try:
-        main()
-    except ExitingException as e:
-        LOG.error("%s: %s", e.__class__.__name__, e)
-        _exit_code = e.exit_code
-    except Exception:
-        LOG.exception('Unexpected error:')
-        _exit_code = 2
-    sys.exit(_exit_code)
+    sys.exit(main())
